function gui_forecast(tabId)
% function gui_shock_decomposition(tabId)
% interface for the DYNARE shock_decomposition command
%
% INPUTS
%   tabId:  GUI tab element which displays shock_decomposition command interface
%
% OUTPUTS
%   none
%
% SPECIAL REQUIREMENTS
%   none

% Copyright (C) 2003-2019 Dynare Team
%
% This file is part of Dynare.
%
% Dynare is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% Dynare is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.

global project_info dynare_gui_ options_ model_settings oo_ M_

bg_color = char(getappdata(0, 'bg_color'));
special_color = char(getappdata(0, 'special_color'));

handles = [];
gui_size = gui_tools.get_gui_elements_size(tabId);

% --- PANELS -------------------------------------
handles.uipanelVars = uipanel( ...
    'Parent', tabId, ...
    'Tag', 'uipanelVars', 'BackgroundColor', special_color,...
    'Units', 'normalized', 'Position', [0.01 0.09 0.48 0.82], ...
    'Title', '', 'BorderType', 'none');

handles = gui_tabs.create_endo_vars(handles, 'forecast');

handles.uipanelResults = uipanel( ...
    'Parent', tabId, ...
    'Tag', 'uipanelVars', 'BackgroundColor', bg_color,...
    'Units', 'normalized', 'Position', [0.51 0.09 0.48 0.82], ...
    'Title', 'Command options:');

uipanelResults_CreateFcn;

% --- STATIC TEXT -------------------------------------

uicontrol( ...
    'Parent', tabId, ...
    'Tag', 'text7', ...
    'Style', 'text', 'BackgroundColor', bg_color,...
    'Units','normalized','Position',[0.01 0.92 0.48 0.05],...
    'FontWeight', 'bold', ...
    'String', 'Select endogenous variables that will be used in forecast:', ...
    'HorizontalAlignment', 'left');

% --- PUSHBUTTONS -------------------------------------
handles.pushbuttonForecast = uicontrol( ...
    'Parent', tabId, ...
    'Tag', 'pushbuttonSimulation', ...
    'Style', 'pushbutton', ...
    'Units','normalized', 'Position', [gui_size.space gui_size.bottom gui_size.button_width gui_size.button_height],...
    'String', 'Forecast', ...
    'Callback', @pushbuttonForecast_Callback);

    function pushbuttonForecast_Callback(hObject, ~)
        if ~(isfield(oo_, 'dr') && isfield(oo_.dr, 'ghu')&& isfield(oo_.dr, 'ghx'))
            gui_tools.show_warning('Please solve the model before running this command (run estimation or stochastic simulation)');
            return
        end
        handles.pushbuttonResults.Enable = 'off';
        old_options = options_;
        old_oo = oo_;
        try
            gui_tools.project_log_entry('Running forecast ', '...');
            [jObj, guiObj] = gui_tools.create_animated_screen('Running forecast, please wait...', tabId);
            if any([handles.endoTable.Data{:, 1}])
                model_settings.varlist_.forecast = handles.endoTable.Data([handles.endoTable.Data{:,1}], 2);
            else
                model_settings.varlist_.forecast = {};
            end

            if handles.smoothed.Value
                if ~isfield(oo_, 'SmoothedVariables')
                    throw(MException('GuiForecast:NoSmoothedVariables', ...
                        'You must first run estimation with the `smoother` option'));
                end
                options_smoother2histval = struct();
                options_smoother2histval.period = str2double(handles.histval.String);
                options_smoother2histval.outfile = [M_.fname '.mat'];
                smoother2histval(options_smoother2histval);
            end

            options_.nodisplay = 0;
            options_.periods = str2double(handles.periods.String);

            M_.endo_histval = [handles.endoTable.Data{:,4}]';
            M_.exo_histval = zeros(size(M_.exo_names));

            oo_.forecast = dyn_forecast(model_settings.varlist_.forecast, M_, options_, oo_, 'simul');

            jObj.stop;
            jObj.setBusyText('All done!');

            handles.pushbuttonResults.Enable = 'on';
            project_info.modified = true;
            project_info.forecast_run = true;
        catch ME
            jObj.stop;
            jObj.setBusyText('Done with errors');
            gui_tools.show_error('Error in execution of forecast command', ME, 'extended');
            uicontrol(hObject);
            oo_ = old_oo;
        end
        options_ = old_options;
        delete(guiObj);
    end

handles.pushbuttonReset = uicontrol( ...
    'Parent', tabId, ...
    'Tag', 'pushbuttonReset', ...
    'Style', 'pushbutton', ...
    'Units','normalized','Position',[gui_size.space*2+gui_size.button_width gui_size.bottom gui_size.button_width gui_size.button_height],...
    'String', 'Reset', ...
    'Callback', @pushbuttonReset_Callback);

    function pushbuttonReset_Callback(~, ~)
        handles.endoTable.Data(:, 1) = {false};
        handles.periods.String = num2str(project_info.default_forecast_periods);
        handles.histval.String = project_info.nobs;
        handles.smoothed = 0;
        handles.select_all_vars.Value = 0;
        handles.consider_only_observed.Value = 0;
    end

handles.pushbuttonClose = uicontrol( ...
    'Parent', tabId, ...
    'Tag', 'pushbuttonReset', ...
    'Style', 'pushbutton', ...
    'Units','normalized','Position',[gui_size.space*3+gui_size.button_width*2 gui_size.bottom gui_size.button_width gui_size.button_height],...
    'String', 'Close this tab', ...
    'Callback',{@close_tab,tabId});

    function close_tab(~, ~, hTab)
        gui_tabs.delete_tab(hTab);
    end

handles.pushbuttonResults = uicontrol( ...
    'Parent', tabId, ...
    'Tag', 'pushbuttonSimulation', ...
    'Style', 'pushbutton', ...
    'Units','normalized','Position',[gui_size.space*4+gui_size.button_width*3 gui_size.bottom gui_size.button_width gui_size.button_height],...
    'String', 'Browse results...', ...
    'Enable', 'On',...
    'Callback', @pushbuttonResults_Callback);

    function pushbuttonResults_Callback(~, ~)
        gui_results('forecast', dynare_gui_.forecast_results);
    end

handles.pushbuttonCloseAll = uicontrol( ...
    'Parent', tabId, ...
    'Tag', 'pushbuttonSimulation', ...
    'Style', 'pushbutton', ...
    'Units','normalized','Position',[gui_size.space*5+gui_size.button_width*4 gui_size.bottom gui_size.button_width gui_size.button_height],...
    'String', 'Close all output figures', ...
    'Enable', 'on',...
    'Callback', @pushbuttonCloseAll_Callback);

    function pushbuttonCloseAll_Callback(~, ~)
        gui_tools.close_all_figures();
    end

    function uipanelResults_CreateFcn()
        top = 1;
        dwidth = gui_size.default_width;
        dheight = gui_size.default_height;
        spc = gui_size.c_width;
        num = 1;
        uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'text7', ...
            'Style', 'text', 'BackgroundColor', bg_color,...
            'Units','normalized','Position',[spc*2 top-num*dheight dwidth*2 dheight/2],...
            'String', 'number of forecast periods:', ...
            'HorizontalAlignment', 'left');

        handles.periods= uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Style', 'edit', 'BackgroundColor', bg_color,...
            'Units','normalized','Position',[spc*3+dwidth*2 top-num*dheight dwidth*1 dheight/2],...
            'HorizontalAlignment', 'left','String', num2str(project_info.default_forecast_periods), ...
            'Callback', {@checkCommOption_Callback,'periods','INTEGER'});
        num = num+1;
        uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'text7', ...
            'Style', 'text', 'BackgroundColor', bg_color,...
            'Units','normalized','Position',[spc*2 top-num*dheight dwidth*2 dheight/2],...
            'String', 'start at historical period:', ...
            'HorizontalAlignment', 'left');

        hvalue = project_info.nobs;
        if ~isnan(options_.nobs)
            hvalue =  options_.nobs;
        end
        handles.histval= uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Style', 'edit', 'BackgroundColor', bg_color,...
            'Units','normalized','Position',[spc*3+dwidth*2 top-num*dheight dwidth*1 dheight/2],...
            'HorizontalAlignment', 'left',...
            'String', hvalue,...
            'Callback', {@checkCommOption_Callback,'histval','INTEGER'});
        
        num = num+1;
        uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'smoothed', ...
            'Style', 'text', 'BackgroundColor', bg_color,...
            'Units','normalized','Position',[spc*2 top-num*dheight dwidth*2 dheight/2],...
            'String', 'Use smoothed values:', ...
            'HorizontalAlignment', 'left');

        handles.smoothed = uicontrol(...
            'Parent', handles.uipanelResults, ...
            'Style','checkbox',...
            'Units','normalized','Position',[spc*3+dwidth*2 top-num*dheight dwidth dheight/2],...
            'TooltipString','Estimation must have been run with `smoother` option.');

        num = num+2;
        uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'text7', ...
            'Style', 'text', 'BackgroundColor', bg_color,...
            'Units','normalized','Position',[spc*2 top-num*dheight dwidth*2 dheight/2],...
            'String', 'consider_all_endogenous:', ...
            'HorizontalAlignment', 'left');

        handles.select_all_vars = uicontrol(...
            'Parent', handles.uipanelResults, ...
            'Style','checkbox',...
            'Units','normalized','Position',[spc*3+dwidth*2 top-num*dheight dwidth dheight/2],...
            'Callback', {@checkCommOption_Callback,'select_all_vars','none'});

        num = num+1;
        uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'text7', ...
            'Style', 'text', 'BackgroundColor', bg_color,...
            'Units','normalized','Position',[spc*2 top-num*dheight dwidth*2 dheight/2],...
            'String', 'consider_only_observed:', ...
            'HorizontalAlignment', 'left');

        handles.consider_only_observed = uicontrol(...
            'Parent', handles.uipanelResults, ...
            'Style','checkbox',...
            'Units','normalized','Position',[spc*3+dwidth*2 top-num*dheight dwidth dheight/2],...
            'TooltipString','Observable variables must be declared.',...
            'Callback', {@checkCommOption_Callback,'consider_only_observed','none'});

        function checkCommOption_Callback(hObject, ~, option_name, ~)
            value = get(hObject, 'Value');
            if strcmp(option_name, 'periods') || strcmp(option_name, 'histval')
                value = get(hObject, 'String');
            end
            switch option_name
                case {'periods', 'histval'}
                    if ~isempty(value) && isnan(str2double(value))
                        gui_tools.show_error([option_name ' must be numeric.']);
                        set(hObject, 'String', '');
                    end
                case 'select_all_vars'
                    if value
                        handles.consider_only_observed.Value = false;
                    end
                    handles.endoTable.Data(:, 1) = {logical(value)};
                case 'consider_only_observed'
                    handles.endoTable.Data(:,1) = {false};
                    if value
                        handles.select_all_vars.Value = false;
                        if isfield(options_, 'varobs')
                            for ii = 1:length(options_.varobs)
                                handles.endoTable.Data(strcmp(options_.varobs{ii}, handles.endoTable.Data(:,2)), 1) = {true};
                            end
                        end
                    end
                otherwise
                    error('forecast options: case not handled.')
            end
        end
    end
end
